<?php
/**
 * Copyright (©) 2021 Pinpoint Designs LTD. All right's reserved.
 *
 * @category    Pinpoint
 * @copyright   Copyright (c) 2021 Pinpoint Designs. (http://www.pinpointdesigns.co.uk)
 * @author      Daniel Cuevas <daniel@pinpointdesigns.co.uk>
 */

namespace Pinpoint\CarouselManager\Model\Stage\Renderer;

use Magento\PageBuilder\Model\Filter\Template;
use Magento\PageBuilder\Model\Stage\HtmlFilter;
use Magento\PageBuilder\Model\Stage\Renderer\WidgetDirective;

/**
 * Class CarouselBlock
 */
class CarouselBlock implements \Magento\PageBuilder\Model\Stage\RendererInterface
{
    /**
     * @var \Pinpoint\CarouselManager\Model\ResourceModel\Carousel\CollectionFactory
     */
    private $carouselCollectionFactory;

    /**
     * @var WidgetDirective
     */
    private $widgetDirectiveRenderer;

    /**
     * @var HtmlFilter
     */
    private $htmlFilter;

    /**
     * @var Template
     */
    private $templateFilter;

    /**
     * @param \Pinpoint\CarouselManager\Model\ResourceModel\Carousel\CollectionFactory $carouselCollectionFactory
     * @param WidgetDirective $widgetDirectiveRenderer
     * @param \Magento\PageBuilder\Model\Stage\HtmlFilter $htmlFilter
     * @param \Magento\PageBuilder\Model\Filter\Template|null $templateFilter
     */
    public function __construct(
        \Pinpoint\CarouselManager\Model\ResourceModel\Carousel\CollectionFactory $carouselCollectionFactory,
        WidgetDirective $widgetDirectiveRenderer,
        HtmlFilter $htmlFilter,
        Template $templateFilter = null
    ) {
        $this->carouselCollectionFactory = $carouselCollectionFactory;
        $this->widgetDirectiveRenderer = $widgetDirectiveRenderer;
        $this->htmlFilter = $htmlFilter;
        $this->templateFilter = $templateFilter ?? \Magento\Framework\App\ObjectManager::getInstance()
                ->get(\Magento\PageBuilder\Model\Filter\Template::class);
    }

    /**
     * Render a state object for the specified block for the stage preview
     *
     * @param array $params
     * @return array
     */
    public function render(array $params): array
    {
        $result = [
            'title' => null,
            'content' => null,
            'error' => null
        ];

        // Short-circuit if needed fields aren't present
        if (empty($params['directive']) && empty($params['carousel_id'])) {
            return $result;
        }

        $collection = $this->carouselCollectionFactory->create();
        $blocks = $collection
            ->addFieldToSelect(['name'])
            ->addFieldToFilter('carousel_id', ['eq' => $params['carousel_id']])
            ->load();

        if ($blocks->count() === 0) {
            $result['error'] = sprintf(__('Block with ID: %s doesn\'t exist'), $params['carousel_id']);

            return $result;
        }

        /**
         * @var \Magento\Cms\Model\Block $block
         */
        $block = $blocks->getFirstItem();
        $result['title'] = $block->getName();

        $result['content'] = $this->htmlFilter->filterHtml(
            $result['title']
        );

        return $result;
    }
}
